if ($args.Length -eq 0)
{
    echo "Usage: make-help.ps1 <input filename> <output filename>"
    exit
}

(Get-Content $args[0]  -encoding UTF8 -Raw) |
    Foreach-Object {$_ -replace '( |td>|<br>|<p>)(https?://[^ \r\n]*[A-Za-z_1-9\/])( |\r|\n|\.)'    , '$1<a href="$2" class="external">$2</a>$3'} |
    Foreach-Object {$_ -replace '( |td>|<br>|<p>)((dbo[1-9]*|xls[1-9]*|doc|logs)\.[A-Za-z_1-9]+)'   , '$1<a href="#$2">$2</a>'} |
    Foreach-Object {$_ -replace '( )(dbo[1-9]*|xls[1-9]*|doc|logs)( |\n)'                           , '$1<a href="#schema.$2">$2</a>$3'} |
    Foreach-Object {$_ -replace '( )(xls_users|xls_developers|xls_formats|xls_admins|doc_readers|doc_writers|log_admins|log_users)' , '$1<a href="#role.$2">$2</a>'} |
    Foreach-Object {$_ -replace '( )(planning_app_admins|planning_app_analysts|planning_app_developers|planning_app_users)'         , '$1<a href="#role.$2">$2</a>'} |
    Foreach-Object {$_ -replace '("#)(xls\.queries)"'                                   , '"https://www.savetodb.com/help/savetodb-framework-views.htm#$2"'} |
    Foreach-Object {$_ -replace '("#)(xls\.view_[a-z_1-9]+)"'                           , '"https://www.savetodb.com/help/savetodb-framework-views.htm#$2"'} |
    Foreach-Object {$_ -replace '("#)(xls\.(usp|xl_)[a-z_1-9]+)"'                       , '"https://www.savetodb.com/help/savetodb-framework-procedures.htm#$2"'} |
    Foreach-Object {$_ -replace '("#)(xls\.get_[a-z_1-9]+)"'                            , '"https://www.savetodb.com/help/savetodb-framework-functions.htm#$2"'} |
    Foreach-Object {$_ -replace '("#)(xls\.[a-z_1-9]+)"'                                , '"https://www.savetodb.com/help/savetodb-framework-tables.htm#$2"'} |
    Foreach-Object {$_ -replace '("#)(schema\.xls|role\.xls_[a-z]+)"'                   , '"https://www.savetodb.com/help/savetodb-framework.htm#$2"'} |
    Foreach-Object {$_ -replace '("#)(doc\.view_[a-z_1-9]+)"'                           , '"https://www.savetodb.com/help/database-help-framework-views.htm#$2"'} |
    Foreach-Object {$_ -replace '("#)(doc\.(usp|xl)_[a-z_1-9]+)"'                       , '"https://www.savetodb.com/help/database-help-framework-procedures.htm#$2"'} |
    Foreach-Object {$_ -replace '("#)(doc\.get_[a-z_1-9]+)"'                            , '"https://www.savetodb.com/help/database-help-framework-functions.htm#$2"'} |
    Foreach-Object {$_ -replace '("#)(doc\.[a-z_1-9]+)"'                                , '"https://www.savetodb.com/help/database-help-framework-tables.htm#$2"'} |
    Foreach-Object {$_ -replace '("#)(schema\.doc|role\.doc_[a-z]+)"'                   , '"https://www.savetodb.com/help/database-help-framework.htm#$2"'} |
    Foreach-Object {$_ -replace '("#)(logs\.view_[a-z_1-9]+)"'                          , '"https://www.savetodb.com/help/change-tracking-framework-views.htm#$2"'} |
    Foreach-Object {$_ -replace '("#)(logs\.(usp|xl)_[a-z_1-9]+)"'                      , '"https://www.savetodb.com/help/change-tracking-framework-procedures.htm#$2"'} |
    Foreach-Object {$_ -replace '("#)(logs\.get_[a-z_1-9]+)"'                           , '"https://www.savetodb.com/help/change-tracking-framework-functions.htm#$2"'} |
    Foreach-Object {$_ -replace '("#)(logs\.[a-z_1-9]+)"'                               , '"https://www.savetodb.com/help/change-tracking-framework-tables.htm#$2"'} |
    Foreach-Object {$_ -replace '("#)(schema\.logs|role\.log_[a-z]+)"'                  , '"https://www.savetodb.com/help/change-tracking-framework.htm#$2"'} |
    Foreach-Object {$_ -replace '("#)((dbo|xls)25\.view_[a-z_1-9]+)"'                   , '"https://www.savetodb.com/help/planning-application-views.htm#$2"'} |
    Foreach-Object {$_ -replace '("#)((dbo|xls)25\.(usp|xl_)[a-z_1-9]+)"'               , '"https://www.savetodb.com/help/planning-application-procedures.htm#$2"'} |
    Foreach-Object {$_ -replace '("#)((dbo|xls)25\.get_[a-z_1-9]+)"'                    , '"https://www.savetodb.com/help/planning-application-functions.htm#$2"'} |
    Foreach-Object {$_ -replace '("#)((dbo|xls)25\.[a-z_1-9]+)"'                        , '"https://www.savetodb.com/help/planning-application-tables.htm#$2"'} |
    Foreach-Object {$_ -replace '("#)(schema\.(dbo25|xls25)|role\.planning_[a-z_]+)"'   , '"https://www.savetodb.com/help/planning-application.htm#$2"'} |
    Foreach-Object {$_ -replace '\*\*([A-Za-z1-9_ ]+)\*\*'                              , '<b>$1</b>'} |
    Foreach-Object {$_ -replace '<p>#### *([^ \n]*)( *)([^ \n]*)( *)([^\n]*?)</p>'      , '<h4 id="$1$3"><a name="$1$3"></a>$1$2$3$4$5</h4>'} |
    Foreach-Object {$_ -replace '<p>### *([^ \n]*)( *)([^ \n]*)( *)([^\n]*?)</p>'       , '<h3 id="$1$3"><a name="$1$3"></a>$1$2$3$4$5</h3>'} |
    Foreach-Object {$_ -replace '<p>```(\r\n)<br>'                                      , '<p><code>$1'} |
    Foreach-Object {$_ -replace '<br>```(\r\n<br>)*</p>'                                , '</code></p>'} |
    Foreach-Object {$_ -replace '(<p>)*!\[(.*)\]\(([A-Za-z_\-1-9\\/:\.]*\.(png|jpg|gif))\)(</p>)*', '<img src="$3" title="$2" alt="$2" />'} |
    Foreach-Object {$_ -replace '\[([^\]\r\n]*)\]\((https?://[^ \)\r\n]*)\)'            , '<a href="$2" class="external">$1</a>'} |
    Foreach-Object {$_ -replace '(framework|application)(\.html?)#(tables|views|procedures|functions)'  , '$1-$3$2#$3'} |
    Set-Content -Encoding UTF8 $args[1]
